from dataclasses import dataclass
from os import stat
from xmlrpc.client import boolean
from fastapi import FastAPI, Response, status, HTTPException, Request
import psycopg2
import  psycopg2.extras
# from user import User
import time
from pydantic import BaseModel
from fastapi.middleware.cors import CORSMiddleware
import snowflake.connector
import datetime
from collections import namedtuple

def create_record(obj, fields):
    ''' given obj from db returns named tuple with fields mapped to values '''
    Record = namedtuple("Record", fields)
    mappings = dict(zip(fields, obj))
    return Record(**mappings)



class User(BaseModel):    
    email: str
    password: str
    

class Project(BaseModel):    
    name: str
    description: str
    shared: bool= True
    created_by: str


app = FastAPI()

origins = [
    
    # "http://localhost.tiangolo.com",
    # "https://localhost.tiangolo.com",
    # "http://localhost",
    # "http://localhost:8080",
    # "http://127.0.0.1:8000"
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

while True:
    try:
        conn = psycopg2.connect(host = 'localhost', 
                                database='postgres', user = 'postgres', password = 'Ayyappa123*')
        cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        print('Database connection was successful')
        break
    except Exception as error:
        print("Database connection failed")
        print("Error", error)
        time.sleep(2)

while True:
    try:
        conn1=snowflake.connector.connect(
        user = 'AMUNDRA',
        password= 'Hariharan123*',
        account ="WG50838.ap-south-1.aws",
        warehouse = "COMPUTE_WH",
        database='FASTAPI',        
        )     
        #create cursor
        curs1=conn1.cursor()
        #execute SQL statement
        # curs1.execute("select current date;")
        #fetch result
        print('Snowflake connection was successful')
        break
    except Exception as error:
        print("Snowflake connection failed")
        print("Error", error)
        time.sleep(2)

@app.post("/users")
def find_user(user: User):
    print(user)
    return {
        "message": 'user found successfully'
    }

@app.post("/projects", status_code= status.HTTP_201_CREATED)
async def create_project(project: Project):
        cursor.execute("INSERT INTO projects(name, description, shared, created_by) VALUES(%s, %s, %s, %s) RETURNING *",
            (project.name, project.description, project.shared, project.created_by))
        new_project = cursor.fetchone()
        conn.commit()
        return {"data" : new_project}

@app.get("/allprojects")
async def get_projects():
    # cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    cursor.execute("SELECT id, name, description, shared FROM projects")
    projects = cursor.fetchall()    
    conn.commit()
    all_projects = []
    for row in projects:
        all_projects.append(dict(row))
    return{"data": all_projects}

@app.post("/suites", status_code= status.HTTP_201_CREATED)
async def create_suite(request: Request):
        # cursor.execute("INSERT INTO projects(name, description, shared, created_by) VALUES(%s, %s, %s, %s) RETURNING *",
        #     (project.name, project.description, project.shared, project.created_by))
        # new_project = cursor.fetchone()
        # conn.commit()
        body = await request.json()
        # print(body)
        cursor.execute("INSERT INTO suites(name, description) VALUES(%s, %s) RETURNING *",
        (body['suitename'], body['suitedescription']))
        new_suite = cursor.fetchone()
        conn.commit()
        suiteid = new_suite[0]    
        i = 0
        for eachiIteration in body['selectcase']:
            if eachiIteration == True:
                cursor.execute("INSERT INTO steps(name, description, type, table_details, suiteid) VALUES(%s, %s, %s, %s, %s) RETURNING *",
                    (body['name'][i], body['description'][i],
                    'generic',
                    body['table'][i],
                    suiteid))
                new_test = cursor.fetchone()
                conn.commit()
                print(new_test)
            i = i + 1
        
        return {"data" : "save successfull"}
@app.get("/alltests")
async def get_tests():
    # cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    cursor.execute("SELECT * FROM steps")
    projects = cursor.fetchall()    
    conn.commit()
    all_tests = []
    for row in projects:
        all_tests.append(dict(row))
    return{"data": all_tests}

@app.delete("/suites/{id}")
async def delete_suite(id: int):
    cursor.execute("DELETE FROM suites WHERE id = %s RETURNING *", (str(id),))
    deleted_id = cursor.fetchone()
   
    if deleted_id == None:
        raise HTTPException(status_code = status.HTTP_404_NOT_FOUND,
                detail = f"Suite id with {id} not found")
    conn.commit()
    cursor.execute("DELETE FROM steps WHERE suiteid = %s RETURNING *", (str(id),))
    deleted_suiteid = cursor.fetchone()
    if deleted_suiteid == None:
        raise HTTPException(status_code = status.HTTP_404_NOT_FOUND,
                detail = f"Suite id with {id} not found")
    conn.commit()
    return {"data" : "suite has been successfully deleted"}

@app.get("/alltests")
async def get_tests():
    # cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    cursor.execute("SELECT * FROM steps")
    projects = cursor.fetchall()    
    conn.commit()
    all_tests = []
    for row in projects:
        all_tests.append(dict(row))
    return{"data": all_tests}

@app.delete("/projects/{id}")
async def delete_project(id: int):
    cursor.execute("DELETE FROM projects WHERE id = %s RETURNING *", (str(id),))
    deleted_id = cursor.fetchone()
   
    if deleted_id == None:
        raise HTTPException(status_code = status.HTTP_404_NOT_FOUND,
                detail = f"Suite id with {id} not found")
    conn.commit()   
    return {"data" : "suite has been successfully deleted"}

@app.get("/testing")
async def execute_testcases():
    conn1=snowflake.connector.connect(
            user = 'ap-south-1.aws',
            password= 'Hariharan123*',
            warehouse = "COMPUTE_WH",
            database='FASTAPI',
            account ="ACCOUNTADMIN",
            schema ="PUBLIC")     
    #create cursor
    curs1=conn1.cursor()
    #execute SQL statement
    curs1.execute("select current date;")
    #fetch result
    print (curs1.fetchone()[0])
    return { 'data': curs1.fetchone()[0]}

@app.get("/executetests/{id}")
async def execute_tests(id: int):
    cursor.execute("SELECT * FROM steps WHERE suiteid = %s", (str(id),))
    tests = cursor.fetchall()
    conn.commit()
    cursor.execute("DELETE FROM results WHERE testsuite_id = %s RETURNING *", (str(id),))
    conn.commit()
    # print(tests)
    all_tests = []
    for row in tests:
        all_tests.append(dict(row))
    
    test_case_details = []
    for test in all_tests:
        
        test_case = test['name'].lower()
        if "record count" in test_case:
           log_msg = []
           message = f'>>> Database connection established {datetime.datetime.now()}'
           log_msg.append(message)
           message = f'>>> User successfully  authenticated {datetime.datetime.now()}'
           log_msg.append(message)
           message = f'>>> record count check execution started at {datetime.datetime.now()}'
           log_msg.append(message)
           message = f'>>> query execution starts for source records -- time: {datetime.datetime.now()}'
           log_msg.append(message)
           cursor.execute("SELECT * FROM employee_demo")
           source_records = cursor.fetchall()
           source_length = len(source_records)
           message = f'>>> length of the suorce records are {len(source_records)} -- time: {datetime.datetime.now()}'
           log_msg.append(message)
           curs1.execute("SELECT * FROM EMPLOYEE_DEMO")
           message = f'>>> query execution starts for target records -- time: {datetime.datetime.now()}'
           log_msg.append(message)
           target_records = curs1.fetchall()
           target_length = len(target_records)
           message = f'>>> length of the target records are {len(target_records)}: -- time: {datetime.datetime.now()}'
           log_msg.append(message)
           message = f'>>> validating the record count source vs target: -- time: {datetime.datetime.now()}'
           log_msg.append(message)
           if source_length == target_length:
            message = f'>>> source and target rowcount is matching:  time: {datetime.datetime.now()}'
            log_msg.append(message)
            message = f'>>> capturing and saving the results to the system'
            log_msg.append(message)
            temp_str = ''
            i = 0
            for log in log_msg:
                if not (i == 0):
                    temp_str = temp_str + ',' + log
                i = i + 1
            
            # cursor.execute("DELETE FROM results WHERE testsuite_id = %s RETURNING *", (str(id),))
            # conn.commit()
            cursor.execute("INSERT INTO results(testcase_name, testsuite_id, testcase_description, project_id, execution_status, log) VALUES(%s, %s, %s, %s, %s, %s) RETURNING *",
            (test['name'], id, test['description'], 'PROJECT_SETUP', 'PASS', temp_str))
            test = cursor.fetchone()
            conn.commit()
            # print(test)
        elif "column checks" in test_case:
            # cursor.execute("DELETE FROM results WHERE testsuite_id = %s RETURNING *", (str(id),))
            # conn.commit()
            cursor.execute("SELECT * FROM employee_demo")  
            col_names_source = [desc[0] for desc in cursor.description] 
            # rows_source = cursor.fetchall()
            # conn.commit()
            # result = []
            # for row in rows_source:
            #     result.append(create_record(row, col_names_source))
            curs1.execute("SELECT * FROM EMPLOYEE_DEMO")
            target_names_source = [desc[0] for desc in curs1.description] 
            conn1.commit()
            # print(col_names_source)
            # print(target_names_source)
            column_check = True
            for source_column, target_column in zip(col_names_source, target_names_source):
                if not source_column.lower() == target_column.lower():
                    column_check = False
            if column_check:
                cursor.execute("INSERT INTO results(testcase_name, testsuite_id, testcase_description, project_id, execution_status, log) VALUES(%s, %s, %s, %s, %s, %s) RETURNING *",
                (test['name'], id, test['description'], 'PROJECT_SETUP', 'PASS', 'Long String'))
                test = cursor.fetchone()
                conn.commit()
        elif "duplicates check" in test_case:
            cursor.execute("SELECT  name, designation, COUNT(*) FROM employee_demo GROUP BY name, designation HAVING COUNT(*) > 1")
            source_dupliactes_count = cursor.fetchall()  
            conn.commit()
            # print(source_dupliactes_count)
            curs1.execute("SELECT  name, designation, COUNT(*) FROM employee_demo GROUP BY name, designation HAVING COUNT(*) > 1")
            target_dupliactes_count = cursor.fetchall()  
            conn1.commit()
            if  len(source_dupliactes_count) == 0 and len(target_dupliactes_count) == 0:
                cursor.execute("INSERT INTO results(testcase_name, testsuite_id, testcase_description, project_id, execution_status, log) VALUES(%s, %s, %s, %s, %s, %s) RETURNING *",
                (test['name'], id, test['description'], 'PROJECT_SETUP', 'PASS', 'Long String'))
                test = cursor.fetchone()
                conn.commit()
    return { 'data': 'results saved successful'}

@app.get("/results")
async def get_tests():
    # cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    cursor.execute("SELECT * FROM results")
    projects = cursor.fetchall()    
    conn.commit()
    all_results = []
    for row in projects:
        all_results.append(dict(row))
    return{"data": all_results}

@app.get("/logs/{testsuite_id}/{id}")
async def get_tests(testsuite_id: int, id: int):
    
    # cur = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
    cursor.execute(f"SELECT log FROM results WHERE testsuite_id = '{str(testsuite_id)}' AND id = '{str(id)}'")
    logs = cursor.fetchall()    
    conn.commit()
    message = logs[0][0]
    # all_tests = []
    # for row in logs:
    #     all_tests.append(dict(row))
    return{message}



    